import json
import logging
from pathlib import Path

import typer
from rich import print

from rh_aws_saml_login import core
from rh_aws_saml_login.utils import blend_text, enable_requests_logging

app = typer.Typer(rich_markup_mode="rich")
BANNER = r"""
         __                                                         __      __            _
   _____/ /_        ____ __      _______      _________ _____ ___  / /     / /___  ____ _(_)___
  / ___/ __ \______/ __ `/ | /| / / ___/_____/ ___/ __ `/ __ `__ \/ /_____/ / __ \/ __ `/ / __ \
 / /  / / / /_____/ /_/ /| |/ |/ (__  )_____(__  ) /_/ / / / / / / /_____/ / /_/ / /_/ / / / / /
/_/  /_/ /_/      \__,_/ |__/|__/____/     /____/\__,_/_/ /_/ /_/_/     /_/\____/\__, /_/_/ /_/
                                                                                /____/
"""
APP_NAME = "rh-aws-saml-login"
APP_DIR = Path(typer.get_app_dir(APP_NAME))
APP_DIR.mkdir(exist_ok=True)
ACCOUNT_CACHE = APP_DIR / "account_cache.json"


def write_accounts_cache(accounts: list[str]) -> None:
    """Write accounts cache to disk."""
    json.dump(accounts, ACCOUNT_CACHE.open("w"))


def read_accounts_cache() -> list[str]:
    """Read accounts cache from disk."""
    if ACCOUNT_CACHE.exists():
        return json.load(ACCOUNT_CACHE.open())
    return []


def complete_account(ctx: typer.Context, incomplete: str):
    names = ctx.params.get("account_name") or []
    for name in read_accounts_cache():
        if name.startswith(incomplete) and name not in names:
            yield name


@app.command(epilog="Made with [red]:heart:[/] by [blue]https://github.com/app-sre[/]")
def cli(  # noqa: PLR0913, PLR0917
    account_name: str = typer.Argument(
        None,
        help="AWS account name. '.' as shortcut to use $AWS_ACCOUNT_NAME.",
        autocompletion=complete_account,
    ),
    region: str = typer.Option("us-east-1", help="AWS region"),
    debug: bool = typer.Option(False, help="Enable debug mode"),
    console: bool = typer.Option(
        False, help="Open the AWS console in browser instead of a local shell"
    ),
    display_banner: bool = typer.Option(True, help="Display a shiny banner"),
    saml_url: str = typer.Option(
        "https://auth.redhat.com/auth/realms/EmployeeIDP/protocol/saml/clients/itaws",
        help="SAML URL",
    ),
):
    """Login to AWS using SAML."""
    logging.basicConfig(
        level=logging.INFO if not debug else logging.DEBUG, format="%(message)s"
    )
    if display_banner:
        print(blend_text(BANNER, (32, 32, 255), (255, 32, 255)))
    if debug:
        enable_requests_logging()
    accounts = core.main(
        account_name=account_name, region=region, console=console, saml_url=saml_url
    )
    write_accounts_cache(accounts)


if __name__ == "__main__":
    app()



import copy
import logging
import os
import subprocess

from rich import print
from rich.text import Text


def blend_text(
    message: str, color1: tuple[int, int, int], color2: tuple[int, int, int]
) -> Text:
    """Blend text from one color to another."""
    text = Text(message)
    r1, g1, b1 = color1
    r2, g2, b2 = color2
    dr = r2 - r1
    dg = g2 - g1
    db = b2 - b1
    size = len(text)
    for index in range(size):
        blend = index / size
        color = f"#{int(r1 + dr * blend):2X}{int(g1 + dg * blend):2X}{int(b1 + db * blend):2X}"
        text.stylize(color, index, index + 1)
    return text


def bye():
    print(
        "Thank you for using rh-aws-saml-login. :man_bowing: Have a great day ahead! :red_heart-emoji:"
    )


def enable_requests_logging():
    from http.client import HTTPConnection  # noqa: PLC0415

    HTTPConnection.debuglevel = 1
    logging.getLogger().setLevel(logging.DEBUG)
    requests_log = logging.getLogger("urllib3")
    requests_log.setLevel(logging.DEBUG)
    requests_log.propagate = True


def run(
    cmd: list[str] | str,
    shell: bool = False,
    check: bool = True,
    capture_output: bool = True,
    env: dict[str, str] | None = None,
) -> subprocess.CompletedProcess:
    shell_env = copy.deepcopy(os.environ)
    if env:
        shell_env.update(env)
    return subprocess.run(
        cmd, shell=shell, check=check, env=shell_env, capture_output=capture_output
    )



import json
import logging
import os
import re
import subprocess
import sys
import urllib
from dataclasses import dataclass
from datetime import datetime as dt
from textwrap import dedent

import boto3
import botocore
import humanize
import requests
from iterfzf import iterfzf
from pyquery import PyQuery as pq
from requests_kerberos import OPTIONAL, HTTPKerberosAuth
from rich import print
from rich.progress import (
    Progress,
    SpinnerColumn,
    TextColumn,
)
from tzlocal import get_localzone

from rh_aws_saml_login.utils import bye, run


@dataclass
class AwsAccount:
    name: str
    uid: str
    role_name: str
    role_arn: str

    @property
    def principle_arn(self) -> str:
        return f"arn:aws:iam::{self.uid}:saml-provider/RedHatInternal"


@dataclass
class AwsCredentials:
    access_key: str
    secret_key: str
    session_token: str
    expiration: dt


def is_kerberos_ticket_valid() -> bool:
    """Test for a valid kerberos ticket."""
    try:
        run(["klist", "--test"], check=True)
        return True
    except subprocess.CalledProcessError:
        return False


def kinit():
    """Acquire a kerberos ticket."""
    run(["kinit"], check=True, capture_output=False)


def get_saml_auth(url: str) -> tuple[str, str]:
    with requests.Session() as session:
        session.auth = HTTPKerberosAuth(mutual_authentication=OPTIONAL)
        r = session.get(url)
        r.raise_for_status()
        p = pq(r.text).xhtml_to_html()
        form = p("form")
        aws_url = form.attr("action")
        saml_token = form("input:hidden").attr("value")
    return aws_url, saml_token


def get_aws_accounts(aws_url: str, saml_token: str) -> list[AwsAccount]:
    r = requests.post(aws_url, data={"SAMLResponse": saml_token})
    r.raise_for_status()
    p = pq(r.text).xhtml_to_html()
    accounts = p("div.saml-account")
    if not accounts:
        logging.error("No AWS accounts found: %s", r.text)
        sys.exit(1)

    aws_accounts = []
    for account in accounts.items():
        name = account.find(".saml-account-name").text()
        if not name:
            continue
        # "Account: foobar (123456789)" or "Account: 123456789"
        name = re.split(r"\s+", name)[1]
        role_labels = account.find(".saml-role").find("label")
        for role_label in role_labels.items():
            # arn:aws:iam::123456789:role/123456789-role-name
            role_arn = role_label.attr("for")
            aws_accounts.append(
                AwsAccount(
                    name=name,
                    uid=role_arn.split(":")[4],
                    role_name=role_label.text(),
                    role_arn=role_arn,
                )
            )
    return aws_accounts


def select_aws_account(
    aws_accounts: list[AwsAccount], account_name: str | None, role: str | None = None
) -> AwsAccount | None:
    if not account_name:
        items = [f"{acc.name:<40} {acc.role_name}" for acc in aws_accounts]
        selected_item = iterfzf(
            items,
            exact=True,
            __extra__=[f"--header={'Account':<40} Role"],
        )
        if not selected_item:
            sys.exit(0)
        account_name, role = re.split(r"\s+", selected_item, maxsplit=1)
    account = next(
        (
            a
            for a in aws_accounts
            if a.name == account_name and (not role or a.role_name == role)
        ),
        None,
    )
    return account


def assume_role_with_saml(account: AwsAccount, saml_token: str) -> AwsCredentials:
    sts = boto3.client(
        "sts", config=botocore.config.Config(signature_version=botocore.UNSIGNED)
    )
    response = sts.assume_role_with_saml(
        RoleArn=account.role_arn,
        PrincipalArn=account.principle_arn,
        SAMLAssertion=saml_token,
    )
    return AwsCredentials(
        access_key=response["Credentials"]["AccessKeyId"],
        secret_key=response["Credentials"]["SecretAccessKey"],
        session_token=response["Credentials"]["SessionToken"],
        expiration=response["Credentials"]["Expiration"],
    )


def open_aws_shell(account: AwsAccount, credentials: AwsCredentials, region: str):
    print(
        dedent(f"""
            Spawning a new shell. Use exit or CTRL+d to leave it!

            :nerd_face: {account.name}
            :rocket: {account.role_name}
            :hourglass: {humanize.naturaltime(credentials.expiration)} ({credentials.expiration.astimezone(tz=get_localzone())})
        """)
    )
    run(
        os.environ["SHELL"],
        check=False,
        capture_output=False,
        env={
            "AWS_ACCOUNT_NAME": account.name,
            "AWS_ROLE_NAME": account.role_name,
            "AWS_ROLE_ARN": account.role_arn,
            "AWS_ACCESS_KEY_ID": credentials.access_key,
            "AWS_SECRET_ACCESS_KEY": credentials.secret_key,
            "AWS_SESSION_TOKEN": credentials.session_token,
            "AWS_REGION": region,
        },
    )


def open_aws_console(account: AwsAccount, credentials: AwsCredentials) -> None:
    """Open the AWS console in a browser.

    See https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_enable-console-custom-url.html
    """
    aws_federated_signin_endpoint = "https://signin.aws.amazon.com/federation"
    # Get a sign-in token from the AWS sign-in federation endpoint.
    response = requests.get(
        aws_federated_signin_endpoint,
        params={
            "Action": "getSigninToken",
            # 12 hours
            "SessionDuration": "43200",
            "Session": json.dumps({
                "sessionId": credentials.access_key,
                "sessionKey": credentials.secret_key,
                "sessionToken": credentials.session_token,
            }),
        },
    )
    signin_token = json.loads(response.text)

    # Make a federated URL that can be used to sign into the AWS Management Console.
    query_string = urllib.parse.urlencode({
        "Action": "login",
        "Issuer": "redhat.com",
        "Destination": "https://console.aws.amazon.com/",
        "SigninToken": signin_token["SigninToken"],
    })
    federated_url = f"{aws_federated_signin_endpoint}?{query_string}"
    run(["open", federated_url], check=False, capture_output=False)


def main(  # noqa: PLR0913, PLR0917
    account_name: str | None, region: str, console: bool, saml_url: str
) -> list[str]:
    with Progress(
        SpinnerColumn(finished_text="✅"),
        TextColumn("[progress.description]{task.description}"),
    ) as progress:
        task = progress.add_task(
            description="Test for a valid Kerberos ticket ...", total=1
        )
        if not is_kerberos_ticket_valid():
            progress.stop()
            logging.info("No valid Kerberos ticket found. Acquiring one ...")
            kinit()
            progress.start()
        progress.update(task, completed=1)

        task = progress.add_task(description="Getting SAML token ...", total=1)
        aws_url, saml_token = get_saml_auth(saml_url)
        progress.update(task, completed=1)

        task = progress.add_task(description="Getting AWS accounts ...", total=1)
        aws_accounts = get_aws_accounts(aws_url, saml_token)

        progress.stop()
        if account_name == ".":
            # account name can be passed as a dot to open the console for the previously selected account
            account_name = os.environ.get("AWS_ACCOUNT_NAME")
        account = select_aws_account(aws_accounts, account_name)
        progress.start()
        if not account:
            logging.error("Account not found: %s", account_name)
            sys.exit(1)

        progress.update(task, completed=1)

        task = progress.add_task(
            description="Getting temporary AWS credentials ...", total=1
        )
        credentials = assume_role_with_saml(account, saml_token)
        progress.update(task, completed=1)

    if console:
        open_aws_console(account, credentials)
    else:
        open_aws_shell(account, credentials, region)
    bye()
    return [acc.name for acc in aws_accounts]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rh_aws_saml_login']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.34.37,<2.0.0',
 'humanize>=4.9.0,<5.0.0',
 'iterfzf>=1.1.0.44.0,<2.0.0.0.0',
 'pyquery>=2.0.0,<3.0.0',
 'requests-kerberos>=0.14.0,<0.15.0',
 'rich>=13.7.0,<14.0.0',
 'typer>=0.9.0,<0.10.0',
 'tzlocal>=5.2,<6.0']

entry_points = \
{'console_scripts': ['rh-aws-saml-login = rh_aws_saml_login.__main__:app']}

setup_kwargs = {
    'name': 'rh-aws-saml-login',
    'version': '0.3.0',
    'description': 'A CLI tool that allows you to log in and retrieve AWS temporary credentials using Red Hat SAML IDP',
    'long_description': '# rh-aws-saml-login\n\n[![Ruff](https://img.shields.io/endpoint?url=https://raw.githubusercontent.com/astral-sh/ruff/main/assets/badge/v2.json)](https://github.com/astral-sh/ruff)\n[![PyPI](https://img.shields.io/pypi/v/rh-aws-saml-login)][pypi-link]\n[![PyPI platforms][pypi-platforms]][pypi-link]\n![PyPI - License](https://img.shields.io/pypi/l/rh-aws-saml-login)\n\nA CLI tool that allows you to log in and retrieve AWS temporary credentials using Red Hat SAML IDP.\n\n![demo](/demo/quickstart.gif)\n\n## Pre-requisites\n\n- Python 3.11 or later\n- Connected to Red Hat VPN\n- A Red Hat managed computer (Kerberos must be installed and configured) and you are logged in with your Red Hat account\n\n## How it works\n\nThe `rh-aws-saml-login` CLI is a tool that simplifies the process of logging into an AWS account via Red Hat SSO. It retrieves a SAML token from the Red Hat SSO server, then fetches and parses the AWS SSO login page to present you with a list of all available accounts and their respective roles. You can then choose your desired account and role, and `rh-aws-saml-login` uses the SAML token to generate temporary AWS role credentials. Finally, it spawns a new shell with the necessary `AWS_` environment variables already set up, so you can immediately use the `aws` CLI without any further configuration.\n\n## Installation\n\nOn CSB Fedora, you need to install the Kerberos development package:\n\n```shell\nsudo dnf install krb5-devel\n```\n\nYou can install this library from [PyPI][pypi-link] with `pip`:\n\n```shell\npython3 -m pip install rh-aws-saml-login\n```\n\nor install it with `pipx`:\n\n```shell\npipx install rh-aws-saml-login\n```\n\nYou can also use `pipx` to run the library without installing it:\n\n```shell\npipx run rh-aws-saml-login\n```\n\n## Usage\n\n```shell\nrh-aws-saml-login\n```\n\nThis spawns a new shell with the following environment variables are set:\n\n- `AWS_ACCOUNT_NAME`: The name/alias of the AWS account\n- `AWS_ROLE_NAME`:  The name of the role\n- `AWS_ROLE_ARN`: The ARN of the role\n- `AWS_ACCESS_KEY_ID`: The access key used by the AWS CLI\n- `AWS_SECRET_ACCESS_KEY`: The secret access key used by the AWS CLI\n- `AWS_SESSION_TOKEN`: The session token used by the AWS CLI\n- `AWS_REGION`: The default region used by the AWS CLI\n\n## Features\n\nrh-aws-saml-login currently provides the following features (get help with `-h` or `--help`):\n\n- No configuration needed\n- Uses Kerberos authentication\n- Open the AWS web console for an account with the `--console` option\n- Shell auto-completion (bash, zsh, and fish) including AWS account names\n- Integrates nicely with the [starship](https://starship.rs)\n\n  ```toml\n   [env_var.AWS_ACCOUNT_NAME]\n   format = "$symbol$style [$env_value]($style) "\n   style = "cyan"\n   symbol = "🚀"\n  ```\n\n## Development\n\n[![Ruff](https://img.shields.io/endpoint?url=https://raw.githubusercontent.com/astral-sh/ruff/main/assets/badge/v2.json)](https://github.com/astral-sh/ruff)\n[![Checked with mypy](https://www.mypy-lang.org/static/mypy_badge.svg)](https://mypy-lang.org/)\n\n- Update CHANGELOG.md with the new version number and date\n- Bump the version number in [pyproject.toml](/pyproject.toml)\n\n[pypi-link]:                https://pypi.org/project/rh-aws-saml-login/\n[pypi-platforms]:           https://img.shields.io/pypi/pyversions/rh-aws-saml-login\n',
    'author': 'Christian Assing',
    'author_email': 'cassing@redhat.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/app-sre/rh-aws-saml-login',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.11',
}


setup(**setup_kwargs)
